function p = PN(par1, addExpnt, minMantLength)
% PN/PN Polynomial Number class constructor: p = PN(par1, addExpnt, minMantLength)
%
% optional addExpnt(=0 by default), means PN exponent if par1 is a vector
% if par1 is a PN or a string then addExpnt is added to the par1.exponent
%
% optional minMantLength = 1 by default
% if (minMantLength > 1) and (minMantLength < length(par1.mant)) then trailing
% zeros are removed
%
% x = PN() creates x=(~0~)
%
% x = PN(y) ,where y is PN, creates x = y
%
% x = PN('...') creates  PN from string like '(~1.1~-2~,3.8~4~)*(~1~0~)^3'
%
% x = PN(mantissa, exponent) creates x=mantissa*(~1~0~)^exponent
%     i.e. PN([a b c d], e) creates (~a~,b~c~d~)*(~1~0~)^e
%     mantissa can be also a column vector ( p.mant = par1(:) )
%     
% x = PN(mantissa, exponent, minMantLength) as above
%     with minimal length of mantissa as minMantLength
%     i.e. PN([a b c d], e, m) creates (~a~,b~c~d~0~0~0~...~)*(~1~0~)^e
%
% x = PN('CONST_NAME', a, minMantLength) creates  PN const, where
% 'CONST_NAME' =  
% 'c_1111'         %   a  * (~1~, 1~1~1~...~)
% 'c_05111'        %   a  * (~0.5~, 1~1~1~...~)
% 'c_P_TRAP'       %  4*a * (~0.5~,-1~1~-1~...~), a=1/h
% 'c_P_PROST'      %   a  * (~1~,-1~), a=1/h
% 'c_P_TRAP_05111' %   a  * (~1~), a=1/h - this is just a scalar
%                                          and minMantLength=1 is foced (!)
%A.Kubaszek, RzUT, 2007-002-12, v.1.0

if ( nargin > 0 )
    if (isa(par1,'PN')) % often used in operator overloading
        p = par1;       % so sould be as early as possible
        if ( nargin == 1 ), return; end
        p.exponent = int64( double(p.exponent) + addExpnt);
        if ( nargin == 2 ), return; end
    end
else
    p.exponent = int64( 0 );
    p.mant = 0;
    p = class(p,'PN');
    return;
end

if ( nargin > 3 ), error('PN constructor error: wrong PN parameters!'); end
if ( nargin < 3 ), minMantLength=1; 
else
    if (isscalar(minMantLength)), minMantLength=round(minMantLength);
    else error('PN constructor error: wrong PN minMantLength!');
    end
end
if ( nargin < 2 ), addExpnt=0; 
else
    if ( ~isscalar(addExpnt) )
        error('PN constructor error: wrong PN exponent!');
    end
end

if (ischar(par1)) %create PN from string like '(~1.1~-2~,3.8~4~)*(~1~0~)^3'
    par1 = strtrim( par1 ); % or as const 'c_...'
    if ( ~strncmp( par1, 'c_', 2 ) )    
        s = par1;
        s = strrep(s, '(', ' ');  s = strrep(s, ')', ' ');
        [mStr, s] = strtok(s, '*');
        p.exponent = int64(0);% p.exponent must be defined before p.mant
        if not (isempty(s)) 
            s = deblank(s);
            [dump, s] = strtok(s, '^');
            s =s(2:end);
            if (isempty(s))
                error('PN constructor error: wrong PN exponent format!')
            end
            p.exponent = int64(str2double(s));
        end

        s = strrep(mStr, '~,', '~*');  % '*' is  unique char
        s = strrep(s, '~', ' ');
        s = deblank(s);
        [mStr, s] = strtok(s, '*');

        m = str2num(mStr); %#ok<ST2NM>
        if (isempty(m))
            error('PN constructor error: wrong PN string format!')
        end
        p.exponent = int64( double(p.exponent) + length(m)-1 + addExpnt);

        if (length(s) > 1)
            s =s(2:end);
            m = [m str2num(s)]; %#ok<ST2NM>
        end

        p.mant = m(1,:).';
        p = class(p,'PN');
    else %create PN constant
        p.exponent = int64(0);
        switch par1     %   a  = addExpnt
        case 'c_1111'   %   a  * (~1~, 1~1~1~... )
                    p.mant = ones(minMantLength,1) * addExpnt;
        case 'c_05111'  %   a  * (~0.5~, 1~1~1~... )
                    p.mant = ones(minMantLength,1) * addExpnt;
                    p.mant(1)=addExpnt/2;
        case 'c_P_TRAP' %  4*a * (~0.5~,-1~1~-1~...) , a=1/h
                    j = 4*addExpnt;
                    p.mant = ones(minMantLength,1) * j;
                    p.mant(1)=2*addExpnt;
                    j=-j;
                    for k=2:2:minMantLength, p.mant(k)=j; end   
        case 'c_P_PROST' %  1*a * (~1~,-1~) , a=1/h
                    p.mant = zeros(minMantLength,1);
                    p.mant(1)=addExpnt; p.mant(2)=-addExpnt;
        case 'c_P_TRAP_05111'  %  a  * (~1~), a=1/h - this is just a scalar
                    p.mant = addExpnt;     % and minMantLength=1 is foced (!)
        otherwise
            error('Unknow constant name %s', par1);
        end 
        p = class(p,'PN');
        return;
    end        
elseif ( ~isa(par1,'PN') ) %create PN from vector
    p.exponent = int64(addExpnt);
    p.mant = par1(:);
    p = class(p,'PN');
end

%AlwaysNormalized conception
p = normalized( p ); 

if ( minMantLength > 1 ) % usage of minMantLength
    a = minMantLength - length(p.mant);
    if ( a > 0 )
        p.mant = [ p.mant; zeros(a,1)];
    else % try to remove trailing zeros
        while ( (p.mant(end) == 0 ) && (length(p.mant) > minMantLength) )
            p.mant(end)= [];
        end
    end
end

end %function
